__author__ = 'chris'
import os
import sys
from django.core.management.base import BaseCommand, CommandError
from django.core.files import File
from django.core.files.storage import default_storage
from django.conf import settings

from ...backend.utils import add_djangui_script
from ... import settings as djangui_settings


class Command(BaseCommand):
    help = 'Closes the specified poll for voting'

    def add_arguments(self, parser):
        parser.add_argument('--group',
            dest='group',
            default='Djangui Scripts',
            help='The name of the group to create scripts under. Default: Djangui Scripts')
        parser.add_argument('script', type=str, help='A script or folder of scripts to add to Djangui.')

    def handle(self, *args, **options):
        script = options['script']
        if not script:
            raise CommandError('You must provide a script path or directory containing scripts.')
        if not os.path.exists(script):
            raise CommandError('{0} does not exist.'.format(script))
        group = options['group'] if options['group'] else 'Djangui Scripts'
        scripts = [os.path.join(script, i) for i in os.listdir(script)] if os.path.isdir(script) else [script]
        converted = 0
        for script in scripts:
            if script.endswith('.pyc') or '__init__' in script:
                continue
            if script.endswith('.py'):
                sys.stdout.write('Converting {}\n'.format(script))
                # copy the script to our storage
                with open(script, 'r') as f:
                    script = default_storage.save(os.path.join(djangui_settings.DJANGUI_SCRIPT_DIR, os.path.split(script)[1]), File(f))
                added, error = add_djangui_script(script=os.path.abspath(os.path.join(settings.MEDIA_ROOT, script)), group=group)
                if added:
                    converted += 1
        sys.stdout.write('Converted {} scripts\n'.format(converted))


from __future__ import absolute_import
from django.contrib.admin import ModelAdmin, site

from djangui.models import Script, ScriptGroup, ScriptParameter, DjanguiJob, ScriptParameterGroup

class JobAdmin(ModelAdmin):
    list_display = ('user', 'job_name', 'script', 'status', 'created_date')

class ScriptAdmin(ModelAdmin):
    list_display = ('script_name', 'script_group', 'is_active', 'script_version')

class ParameterAdmin(ModelAdmin):
    list_display = ('script', 'parameter_group', 'short_param')

class GroupAdmin(ModelAdmin):
    list_display = ('group_name', 'is_active')

class ParameterGroupAdmin(ModelAdmin):
    list_display = ('script', 'group_name')

site.register(DjanguiJob, JobAdmin)
site.register(Script, ScriptAdmin)
site.register(ScriptParameter, ParameterAdmin)
site.register(ScriptGroup, GroupAdmin)
site.register(ScriptParameterGroup, ParameterGroupAdmin)


# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('djangui', '0003_auto_20150507_0347'),
    ]

    operations = [
        migrations.CreateModel(
            name='DjanguiFile',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('filepath', models.FileField(upload_to=b'')),
                ('filepreview', models.TextField(null=True, blank=True)),
                ('filetype', models.CharField(max_length=255, null=True, blank=True)),
                ('job', models.ForeignKey(to='djangui.DjanguiJob')),
                ('parameter', models.ForeignKey(blank=True, to='djangui.ScriptParameters', null=True)),
            ],
        ),
    ]



from __future__ import absolute_import
__author__ = 'chris'
import os
import errno
import importlib
import json

from django.core.files.storage import default_storage
from django.core.files.base import ContentFile
from django.db import models
from django.conf import settings
from django.core.urlresolvers import reverse_lazy, reverse
from django.contrib.auth.models import Group
from django.utils.translation import gettext_lazy as _
from django.db import transaction
from django.utils.text import get_valid_filename

from autoslug import AutoSlugField

from celery import states

from .. import settings as djangui_settings
from .. backend import utils

from . mixins import UpdateScriptsMixin, ModelDiffMixin


# TODO: Handle cases where celery is not setup but specified to be used
tasks = importlib.import_module(djangui_settings.DJANGUI_CELERY_TASKS)

class ScriptGroup(UpdateScriptsMixin, models.Model):
    """
        This is a group of scripts, it holds general information
        about a collection of scripts, and allows for custom descriptions

    """
    group_name = models.TextField()
    slug = AutoSlugField(populate_from='group_name', unique=True)
    group_description = models.TextField(null=True, blank=True)
    group_order = models.SmallIntegerField(default=1)
    is_active = models.BooleanField(default=True)
    user_groups = models.ManyToManyField(Group, blank=True)

    def __unicode__(self):
        return unicode(self.group_name)

class Script(ModelDiffMixin, models.Model):
    script_name = models.CharField(max_length=255)
    slug = AutoSlugField(populate_from='script_name', unique=True)
    script_group = models.ForeignKey('ScriptGroup')
    script_description = models.TextField(blank=True, null=True)
    script_order = models.PositiveSmallIntegerField(default=1)
    is_active = models.BooleanField(default=True)
    user_groups = models.ManyToManyField(Group, blank=True)
    script_path = models.FileField()
    execute_full_path = models.BooleanField(default=True) # use full path for subprocess calls
    save_path = models.CharField(max_length=255, blank=True, null=True,
                                 help_text='By default save to the script name,'
                                           ' this will change the output folder.')
    # when a script updates, increment this to keep old scripts that are cloned working. The downside is we get redundant
    # parameters, but even a huge site may only have a few thousand parameters to query though.
    script_version = models.PositiveSmallIntegerField(default=0)

    created_date = models.DateTimeField(auto_now_add=True)
    modified_date = models.DateTimeField(auto_now=True)

    def __unicode__(self):
        return unicode(self.script_name)

    def get_url(self):
        return reverse('djangui_script', kwargs={'script_group': self.script_group.slug,
                                                      'script_name': self.slug})

    def get_script_path(self):
        path = self.script_path.path
        return path if self.execute_full_path else os.path.split(path)[1]

    @transaction.atomic
    def save(self, **kwargs):
        if 'script_path' in self.changed_fields:
            self.script_version += 1
        new_script = self.pk is None or 'script_path' in self.changed_fields
        # if uploading from the admin, fix its path
        # we do this to avoid having migrations specific to various users with different DJANGUI_SCRIPT_DIR settings
        if new_script or djangui_settings.DJANGUI_SCRIPT_DIR not in self.script_path.file.name:
            new_name = os.path.join(djangui_settings.DJANGUI_SCRIPT_DIR, self.script_path.file.name)
            utils.get_storage(local=False).save(new_name, self.script_path.file)
            # save it locally a well
            if not utils.get_storage(local=True).exists(new_name):
                utils.get_storage(local=True).save(new_name, self.script_path.file)
            self.script_path.save(new_name, self.script_path.file, save=False)
            self.script_path.name = new_name
        super(Script, self).save(**kwargs)
        if new_script:
            if getattr(self, '_add_script', True):
                added, error = utils.add_djangui_script(script=self, group=self.script_group)
                if added is False:
                    # TODO: Make a better error
                    raise BaseException(error)
        utils.load_scripts()



class DjanguiJob(models.Model):
    """
    This model serves to link the submitted celery tasks to a script submitted
    """
    # blank=True, null=True is to allow anonymous users to submit jobs
    user = models.ForeignKey(settings.AUTH_USER_MODEL, blank=True, null=True)
    celery_id = models.CharField(max_length=255, null=True)
    job_name = models.CharField(max_length=255)
    job_description = models.TextField(null=True, blank=True)
    stdout = models.TextField(null=True, blank=True)
    stderr = models.TextField(null=True, blank=True)

    DELETED = 'deleted'
    SUBMITTED = 'submitted'
    COMPLETED = 'completed'
    RUNNING = 'running'

    STATUS_CHOICES = (
        (SUBMITTED, _('Submitted')),
        (RUNNING, _('Running')),
        (COMPLETED, _('Completed')),
        (DELETED, _('Deleted')),
    )

    status = models.CharField(max_length=255, default=SUBMITTED, choices=STATUS_CHOICES)

    save_path = models.CharField(max_length=255, blank=True, null=True)
    command = models.TextField()
    created_date = models.DateTimeField(auto_now_add=True)
    modified_date = models.DateTimeField(auto_now=True)
    script = models.ForeignKey('Script')

    def __unicode__(self):
        return unicode(self.job_name)

    def get_parameters(self):
        return ScriptParameters.objects.filter(job=self)

    def submit_to_celery(self, **kwargs):
        if kwargs.get('resubmit'):
            params = self.get_parameters()
            self.pk = None
            self.save()
            with transaction.atomic():
                for param in params:
                    param.pk = None
                    param.job = self
                    param.recreate()
                    param.save()
        self.status = self.SUBMITTED
        self.save()
        if djangui_settings.DJANGUI_CELERY:
            results = tasks.submit_script.delay(djangui_job=self.pk)
        else:
            results = tasks.submit_script(djangui_job=self.pk)
        return self

    def get_resubmit_url(self):
        return reverse('djangui_script_clone', kwargs={'script_group': self.script.script_group.slug,
                                                      'script_name': self.script.slug, 'job_id': self.pk})

    @staticmethod
    def mkdirs(path):
        try:
            os.makedirs(path)
        except OSError as exc:
            if exc.errno == errno.EEXIST and os.path.isdir(path):
                pass
            else:
                raise

    def get_output_path(self):
        path = os.path.join(djangui_settings.DJANGUI_FILE_DIR, get_valid_filename(self.user.username if self.user is not None else ''),
                            get_valid_filename(self.script.slug if not self.script.save_path else self.script.save_path), str(self.pk))
        self.mkdirs(os.path.join(settings.MEDIA_ROOT, path))
        return path

    def get_upload_path(self):
        path = os.path.join(djangui_settings.DJANGUI_FILE_DIR, get_valid_filename(self.user.username if self.user is not None else ''),
                            get_valid_filename(self.script.slug if not self.script.save_path else self.script.save_path))
        self.mkdirs(os.path.join(settings.MEDIA_ROOT, path))
        return path


class ScriptParameterGroup(UpdateScriptsMixin, models.Model):
    group_name = models.TextField()
    script = models.ForeignKey('Script')

    def __unicode__(self):
        return unicode('{}: {}'.format(self.script.script_name, self.group_name))


class ScriptParameter(UpdateScriptsMixin, models.Model):
    """
        This holds the parameter mapping for each script, and enforces uniqueness by each script via a FK.
    """
    script = models.ForeignKey('Script')
    short_param = models.CharField(max_length=255)
    script_param = models.CharField(max_length=255)
    slug = AutoSlugField(populate_from='script_param', unique=True)
    is_output = models.BooleanField(default=None)
    required = models.BooleanField(default=False)
    output_path = models.FilePathField(path=settings.MEDIA_ROOT, allow_folders=True, allow_files=False,
                                       recursive=True, max_length=255)
    choices = models.CharField(max_length=255, null=True, blank=True)
    choice_limit = models.PositiveSmallIntegerField(null=True, blank=True)
    form_field = models.CharField(max_length=255)
    default = models.CharField(max_length=255, null=True, blank=True)
    input_type = models.CharField(max_length=255)
    param_help = models.TextField(verbose_name='help', null=True, blank=True)
    is_checked = models.BooleanField(default=False)
    parameter_group = models.ForeignKey('ScriptParameterGroup')

    def __unicode__(self):
        return unicode('{}: {}'.format(self.script.script_name, self.script_param))


# TODO: find a better name for this class
class ScriptParameters(models.Model):
    """
        This holds the actual parameters sent with the submission
    """
    # the details of the actual executed scripts
    job = models.ForeignKey('DjanguiJob')
    parameter = models.ForeignKey('ScriptParameter')
    # we store a JSON dumped string in here to attempt to keep our types in order
    _value = models.TextField(db_column='value')

    BOOLEAN = 'BooleanField'
    CHAR = 'CharField'
    CHOICE = 'ChoiceField'
    FILE = 'FileField'
    FLOAT = 'FloatField'
    INTEGER = 'IntegerField'

    DJANGUI_FIELD_MAP = {
        BOOLEAN: lambda x: str(x).lower() == 'true',
        CHAR: str,
        CHOICE: str,
        FILE: file,
        FLOAT: float,
        INTEGER: int,
    }

    def __unicode__(self):
        return unicode('{}: {}'.format(self.parameter.script_param, self.value))

    def get_subprocess_value(self):
        value = self.value
        if self.value is None:
            return []
        field = self.parameter.form_field
        param = self.parameter.short_param
        if field == self.BOOLEAN:
            if value:
                return [param]
        if field == self.FILE:
            if self.parameter.is_output:
                try:
                    value = value.path
                except AttributeError:
                    value = utils.get_storage(local=True).path(value)
                    # trim the output path, we don't want to be adding our platform specific paths to the output
                    op = self.job.get_output_path()
                    value = value[value.find(op)+len(op)+1:]
            else:
                # make sure we have it locally otherwise download it
                if not utils.get_storage(local=True).exists(value.path):
                    new_path = utils.get_storage(local=True).save(value.path, value)
                    value = new_path
                else:
                    # return the string for processing
                    value = value.path
        return [param, str(value)]

    def force_value(self, value):
        self._value = json.dumps(value)

    def recreate(self):
        # we want to change filefields to reflect whatever is the current job's path. This is currently used for
        # job resubmission
        value = json.loads(self._value)
        field = self.parameter.form_field
        if field == self.FILE:
            # we are perfectly fine using old input files instead of recreating them, so only check output files
            if self.parameter.is_output:
                new_path = self.job.get_output_path()
                new_root, new_id = os.path.split(new_path)
                # we want to remove the root + the old job's pk
                value = value[value.find(new_root)+len(new_root)+1:]
                value = value[value.find(os.path.sep)+1:]
                # we want to create a new path for the current job
                path = os.path.join(new_path, self.parameter.slug if not value else value)
                value = path
                self._value = json.dumps(value)

    @property
    def value(self):
        value = json.loads(self._value)
        if value is not None:
            field = self.parameter.form_field
            if field == self.FILE:
                try:
                    file_obj = utils.get_storage_object(value)
                    value = file_obj
                except IOError:
                    # this can occur when the storage object is not yet made for output
                    if self.parameter.is_output:
                        return value
                    raise IOError
        return value

    @value.setter
    def value(self, value):
        # coerce the value to the proper type and store as json to make it persistent as well as have json
        #  handle type conversion on the way back out
        field = self.parameter.form_field
        if field == self.CHAR:
            if value is None:
                value = None
            elif field == self.CHAR:
                if not value:
                    value = None
            else:
                value = self.DJANGUI_FIELD_MAP[field](value)
        elif field == self.INTEGER:
            value = self.DJANGUI_FIELD_MAP[field](value) if isinstance(value, int) or str(value).isdigit() else None
        elif field == self.BOOLEAN:
            if value is None or value is False:
                value = None
            if value:
                value = True
        elif field == self.FILE:
            if self.parameter.is_output:
                # make a fake object for it
                path = os.path.join(self.job.get_output_path(), self.parameter.slug if not value else value)
                value = path
            else:
                if value:
                    path = os.path.join(self.job.get_upload_path(), os.path.split(value.name)[1])
                    utils.get_storage(local=False).save(path, value)
                    utils.get_storage(local=True).save(path, value)
                    value = path
        self._value = json.dumps(value)


class DjanguiFile(models.Model):
    filepath = models.FileField()
    job = models.ForeignKey('DjanguiJob')
    filepreview = models.TextField(null=True, blank=True)
    filetype = models.CharField(max_length=255, null=True, blank=True)
    parameter = models.ForeignKey('ScriptParameters', null=True, blank=True)



from __future__ import absolute_import
from django.views.generic import DetailView, TemplateView, CreateView
from django.http import JsonResponse
from django.conf import settings
from django.forms import FileField
from django.core.files.storage import default_storage
from django.utils.translation import gettext_lazy as _
from django.utils.encoding import force_unicode

from djangui.backend import utils
from ..models import DjanguiJob, Script
from .. import settings as djangui_settings
from ..templatetags.djangui_tags import valid_user


class DjanguiScriptJSON(DetailView):
    model = Script
    slug_field = 'slug'
    slug_url_kwarg = 'script_name'

    def render_to_response(self, context, **response_kwargs):
        # returns the models required and optional fields as html
        job_id = self.kwargs.get('job_id')
        initial = None
        if job_id:
            job = DjanguiJob.objects.get(pk=job_id)
            if job.user is None or (self.request.user.is_authenticated() and job.user == self.request.user):
                initial = {}
                for i in job.get_parameters():
                    value = i.value
                    if value is not None:
                        initial[i.parameter.slug] = value
        d = utils.get_form_groups(model=self.object, initial=initial)
        return JsonResponse(d)

    def post(self, request, *args, **kwargs):
        post = request.POST.copy()
        if request.user.is_authenticated() or not djangui_settings.DJANGUI_ALLOW_ANONYMOUS:
            post['user'] = request.user
        form = utils.get_master_form(pk=post['djangui_type'])
        # TODO: Check with people who know more if there's a smarter way to do this
        utils.validate_form(form=form, data=post, files=request.FILES)
        # for cloned jobs, we don't have the files in input fields, they'll be in a list like ['', filename]
        # This will cause issues.
        to_delete = []
        for i in post:
            if isinstance(form.fields.get(i), FileField):
                # if we have a value set, reassert this
                new_value = post.get(i)
                if i not in request.FILES and (i not in form.cleaned_data or (not form.cleaned_data[i] and new_value)):
                    # this is a previously set field, so a cloned job
                    if new_value is not None:
                        form.cleaned_data[i] = utils.get_storage(local=False).open(new_value)
                    to_delete.append(i)
        for i in to_delete:
            if i in form.errors:
                del form.errors[i]

        if not form.errors:
            # data = form.cleaned_data
            script = Script.objects.get(pk=form.cleaned_data.get('djangui_type'))
            valid = valid_user(script, request.user).get('valid')
            if valid is True:
                group_valid = valid_user(script.script_group, request.user).get('valid')
                if valid is True and group_valid is True:
                    job = form.save()
                    job.submit_to_celery()
                    return JsonResponse({'valid': True})
            return JsonResponse({'valid': False, 'errors': {'__all__': [force_unicode(_('You are not permitted to access this script.'))]}})
        return JsonResponse({'valid': False, 'errors': form.errors})


class DjanguiHomeView(TemplateView):
    template_name = 'djangui_home.html'

    def get_context_data(self, **kwargs):
        job_id = self.request.GET.get('job_id')
        ctx = super(DjanguiHomeView, self).get_context_data(**kwargs)
        ctx['djangui_scripts'] = getattr(settings, 'DJANGUI_SCRIPTS', {})
        if job_id:
            job = DjanguiJob.objects.get(pk=job_id)
            if job.user is None or (self.request.user.is_authenticated() and job.user == self.request.user):
                ctx['clone_job'] = {'job_id': job_id, 'url': job.get_resubmit_url(), 'data_url': job.script.get_url()}
        return ctx


import os

from django.test import TestCase

from ..backend import utils

from . import factories
from . import config

class TestUtils(TestCase):
    def test_sanitize_name(self):
        assert(utils.sanitize_name('abc')) == 'abc'
        assert(utils.sanitize_name('ab c')) == 'ab_c'
        assert(utils.sanitize_name('ab-c')) == 'ab_c'

    def test_sanitize_string(self):
        assert(utils.sanitize_string('ab"c')) == 'ab\\"c'

    def test_add_script(self):
        pass
        # TODO: fix me
        # utils.add_djangui_script(script=os.path.join(config.DJANGUI_TEST_SCRIPTS, 'translate.py'))


from __future__ import absolute_import

from django.conf.urls import include, url
from django.conf import settings
from django.conf.urls.static import static

# from djangui.admin import
from djangui.views import celery_status, CeleryTaskView, celery_task_command, DjanguiScriptJSON, DjanguiHomeView, DjanguiRegister, djangui_login

from . import settings as djangui_settings


urlpatterns = [
    url(r'^celery/command$', celery_task_command, name='celery_task_command'),
    url(r'^celery/status$', celery_status, name='celery_results'),
    url(r'^celery/(?P<job_id>[a-zA-Z0-9\-]+)/$', CeleryTaskView.as_view(), name='celery_results_info'),
    # url(r'^admin/', include(djangui_admin.urls)),
    url(r'^djscript/(?P<script_group>[a-zA-Z0-9\-\_]+)/(?P<script_name>[a-zA-Z0-9\-\_]+)/(?P<job_id>[a-zA-Z0-9\-]+)$',
        DjanguiScriptJSON.as_view(), name='djangui_script_clone'),
    url(r'^djscript/(?P<script_group>[a-zA-Z0-9\-\_]+)/(?P<script_name>[a-zA-Z0-9\-\_]+)/$', DjanguiScriptJSON.as_view(), name='djangui_script'),
    url(r'^$', DjanguiHomeView.as_view(), name='djangui_home'),
    url(r'^$', DjanguiHomeView.as_view(), name='djangui_task_launcher'),
    url('^{}'.format(djangui_settings.DJANGUI_LOGIN_URL), djangui_login, name='djangui_login'),
    url('^{}'.format(djangui_settings.DJANGUI_REGISTER_URL), DjanguiRegister.as_view(), name='djangui_register'),
    url('^', include('django.contrib.auth.urls')),
]


from __future__ import absolute_import
import os

from django.http import JsonResponse
from django.core.urlresolvers import reverse
from django.views.generic import TemplateView
from django.conf import settings
from django.utils.translation import gettext_lazy as _
from django.utils.encoding import force_unicode

from djcelery.models import TaskMeta
from celery import app, states

celery_app = app.app_or_default()

from ..models import DjanguiJob
from .. import settings as djangui_settings

def celery_status(request):
    spanbase = "<span class='glyphicon {}' data-toggle='tooltip' data-trigger='hover' title='{}'></span>"
    STATE_MAPPER = {
        DjanguiJob.COMPLETED: spanbase.format('glyphicon-ok', _('Success')),
        DjanguiJob.RUNNING: spanbase.format('glyphicon-refresh spinning', _('Executing')),
        states.PENDING: spanbase.format('glyphicon-time', _('In queue')),
        states.REVOKED: spanbase.format('glyphicon-stop', _('Halted')),
        DjanguiJob.SUBMITTED: spanbase.format('glyphicon-hourglass', _('Waiting to be queued'))
    }
    jobs = DjanguiJob.objects.filter(user=request.user if request.user.is_authenticated() else None).exclude(status=DjanguiJob.DELETED)
    return JsonResponse([{'job_name': job.job_name, 'job_status': STATE_MAPPER.get(job.status, job.status),
                        'job_submitted': job.created_date.strftime('%b %d %Y, %H:%M:%S'),
                        'job_id': job.pk, 'job_description': 'Script: {}\n{}'.format(job.script.script_name, job.job_description),
                        'job_url': reverse('celery_results_info', kwargs={'job_id': job.pk})} for job in jobs], safe=False)


def celery_task_command(request):

    command = request.POST.get('celery-command')
    job_id = request.POST.get('job-id')
    job = DjanguiJob.objects.get(pk=job_id)
    from ..backend.utils import valid_user
    response = {'valid': False,}
    valid = valid_user(job.script, request.user)
    if valid.get('valid') is True:
        user = request.user if request.user.is_authenticated() else None
        if user == job.user:
            if command == 'resubmit':
                new_job = job.submit_to_celery(resubmit=True)
                response.update({'valid': True, 'extra': {'task_url': reverse('celery_results_info', kwargs={'job_id': new_job.pk})}})
            elif command == 'clone':
                response.update({'valid': True, 'redirect': '{0}?job_id={1}'.format(reverse('djangui_task_launcher'), job_id)})
            elif command == 'delete':
                job.status = DjanguiJob.DELETED
                job.save()
                response.update({'valid': True, 'redirect': reverse('djangui_home')})
            elif command == 'stop':
                celery_app.control.revoke(job.celery_id, signal='SIGKILL', terminate=True)
                job.status = states.REVOKED
                job.save()
                response.update({'valid': True, 'redirect': reverse('celery_results_info', kwargs={'job_id': job_id})})
            else:
                response.update({'errors': {'__all__': [force_unicode(_("Unknown Command"))]}})
    else:
        response.update({'errors': {'__all__': [force_unicode(valid.get('error'))]}})
    return JsonResponse(response)


class CeleryTaskView(TemplateView):
    template_name = 'tasks/task_view.html'

    def get_context_data(self, **kwargs):
        ctx = super(CeleryTaskView, self).get_context_data(**kwargs)
        job_id = ctx.get('job_id')
        djangui_job = DjanguiJob.objects.get(pk=job_id)
        ctx['task_info'] = {'stdout': '', 'stderr': '', 'job': djangui_job,
                            'all_files': {},
                            'file_groups': {}}
        from ..backend import utils
        out_files = utils.get_file_previews(djangui_job)
        all = out_files.pop('all', [])
        archives = out_files.pop('archives', [])
        ctx['task_info'].update({
                'all_files': all,
                'archives': archives,
                'file_groups': out_files,
                'status': djangui_job.status,
                'last_modified': djangui_job.modified_date,
            })
        return ctx




from os import environ
from .djangui_settings import *

# This file is where the user can override and customize their installation of djangui

# Djangui Apps - add additional apps here after the initial install (remember to follow everything by a comma)

INSTALLED_APPS += (
)

# Whether to allow anonymous job submissions, set False to disallow 'guest' job submissions
DJANGUI_ALLOW_ANONYMOUS = True

## Celery related options
INSTALLED_APPS += (
    'djcelery',
    'kombu.transport.django',
)

CELERY_RESULT_BACKEND='djcelery.backends.database:DatabaseBackend'
BROKER_URL = 'django://'
CELERY_TRACK_STARTED = True
DJANGUI_CELERY = True
CELERY_SEND_EVENTS = True

# Things you most likely do not need to change

# the directory for uploads (physical directory)
MEDIA_ROOT = os.path.join(BASE_DIR, 'user_uploads')
# the url mapping
MEDIA_URL = '/uploads/'

# the directory to store our webpage assets (images, javascript, etc.)
STATIC_ROOT = os.path.join(BASE_DIR, 'static')
# the url mapping
STATIC_URL = '/static/'

## Here is a setup example for production servers

## A postgres database -- for multiple users a sqlite based database is asking for trouble

# DATABASES = {
#     'default': {
#         'ENGINE': 'django.db.backends.postgresql_psycopg2',
#         # for production environments, these should be stored as environment variables
#         # I also recommend the django-heroku-postgresify package for a super simple setup
#         'NAME': os.environ.get('DATABASE_NAME', 'djangui'),
#         'USER': os.environ.get('DATABASE_USER', 'djangui'),
#         'PASSWORD': os.environ.get('DATABASE_PASSWORD', 'djangui'),
#         'HOST': os.environ.get('DATABASE_URL', 'localhost'),
#         'PORT': os.environ.get('DATABASE_PORT', '5432')
#     }
# }

## A better celery backend -- using RabbitMQ (these defaults are from two free rabbitmq Heroku providers)
# CELERY_RESULT_BACKEND = 'amqp'
# BROKER_URL = os.environ.get('AMQP_URL') or \
#              os.environ.get('RABBITMQ_BIGWIG_TX_URL') or \
#              os.environ.get('CLOUDAMQP_URL', 'amqp://guest:guest@localhost:5672/')
# BROKER_POOL_LIMIT = 1
# CELERYD_CONCURRENCY = 1
# CELERY_TASK_SERIALIZER = 'json'
# ACKS_LATE = True
#

## for production environments, django-storages abstracts away much of the difficulty of various storage engines.
## Here is an example for hosting static and user generated content with S3

# from boto.s3.connection import VHostCallingFormat
#
# INSTALLED_APPS += (
#     'storages',
#     'collectfast',
# )

## We have user authentication -- we need to use https (django-sslify)
# if not DEBUG:
#     MIDDLEWARE_CLASSES = ['sslify.middleware.SSLifyMiddleware']+list(MIDDLEWARE_CLASSES)
#     SECURE_PROXY_SSL_HEADER = ('HTTP_X_FORWARDED_PROTO', 'https')
#
# ALLOWED_HOSTS = (
#     'localhost',
#     '127.0.0.1',
#     "djangui.herokuapp.com",# put your site here
# )
#
# AWS_CALLING_FORMAT = VHostCallingFormat
#
# AWS_ACCESS_KEY_ID = environ.get('AWS_ACCESS_KEY_ID', '')
# AWS_SECRET_ACCESS_KEY = environ.get('AWS_SECRET_ACCESS_KEY', '')
# AWS_STORAGE_BUCKET_NAME = environ.get('AWS_STORAGE_BUCKET_NAME', '')
# AWS_AUTO_CREATE_BUCKET = True
# AWS_QUERYSTRING_AUTH = False
# AWS_S3_SECURE_URLS = True
# AWS_FILE_OVERWRITE = False
# AWS_PRELOAD_METADATA = True
# AWS_S3_CUSTOM_DOMAIN = environ.get('AWS_S3_CUSTOM_DOMAIN', '')
#
# GZIP_CONTENT_TYPES = (
#     'text/css',
#     'application/javascript',
#     'application/x-javascript',
#     'text/javascript',
# )
#
# AWS_EXPIREY = 60 * 60 * 7
# AWS_HEADERS = {
#     'Cache-Control': 'max-age=%d, s-maxage=%d, must-revalidate' % (AWS_EXPIREY,
#         AWS_EXPIREY)
# }
#
# STATIC_URL = 'http://%s.s3.amazonaws.com/' % AWS_STORAGE_BUCKET_NAME
# MEDIA_URL = '/user-uploads/'
#
# STATICFILES_STORAGE = DEFAULT_FILE_STORAGE = 'djangui.djanguistorage.CachedS3BotoStorage'
# DJANGUI_EPHEMERAL_FILES = True


AUTHENTICATION_BACKEND = 'django.contrib.auth.backends.ModelBackend'


__author__ = 'chris'
import traceback
import sys

class ProcessExceptionMiddleware(object):
    def process_response(self, request, response):
        if response.status_code != 200:
            print '\n'.join(traceback.format_exception(*sys.exc_info()))
        return response


from .django_settings import *

INSTALLED_APPS += (
    # 'corsheaders',
    'djangui',
)

# MIDDLEWARE_CLASSES = [[i] if i == 'django.middleware.common.CommonMiddleware' else ['corsheaders.middleware.CorsMiddleware',i] for i in MIDDLEWARE_CLASSES]
MIDDLEWARE_CLASSES = list(MIDDLEWARE_CLASSES)
MIDDLEWARE_CLASSES.append('{{ project_name }}.middleware.ProcessExceptionMiddleware')

PROJECT_NAME = "{{ project_name }}"
DJANGUI_CELERY_APP_NAME = 'djangui.celery'
DJANGUI_CELERY_TASKS = 'djangui.tasks'



from __future__ import absolute_import
__author__ = 'chris'
import argparse
import sys
import copy
import traceback
import os
from django.db.models import Model
from . import utils

# these values cannot be field names assigned by a script, for instances such as a script having a parameter
# 'save'
FORBIDDEN = set(dir(Model))

def filetype_filefield_default(kwargs, attr, default_kwargs, action, extra=None):
    model_name = default_kwargs['model_name']
    value = getattr(action, attr)
    # import pdb; pdb.set_trace();
    if value in (None, sys.stderr, sys.stdout, os.devnull):
        # For files we are saving (out script output), handle cases where they are
        # going to stdout/etc.
        if not is_upload(action):
            if not action.required:
                kwargs['blank'] = True
            if value == sys.stderr:
                kwargs['djangui_output_default'] = "stderr"
            elif value == sys.stdout:
                kwargs['djangui_output_default'] = "stdout"
            elif value == os.devnull or value is None:
                kwargs['djangui_output_default'] = "null"
        model_name = 'upload_to'
        value = ['user_upload' if is_upload(action) else 'user_output']
        value.append("" if extra is None else extra.get('class_name', ''))
        value = "'{0}'".format(os.path.join(*value))
    kwargs[model_name] = value

def filetype_filefield_blank(kwargs, attr, default_kwargs, action, extra=None):
    model_name = default_kwargs['model_name']
    default = action.default
    if is_upload(action) and default == sys.stdin:
        kwargs[model_name] = False

def str_charfield_default(kwargs, attr, default_kwargs, action, extra=None):
    model_name = default_kwargs['model_name']
    value = getattr(action, attr)
    if value is not None:
        kwargs[model_name] = '"{0}"'.format(value)

def charfield_choices(kwargs, attr, default_kwargs, action, extra=None):
    model_name = default_kwargs['model_name']
    value = getattr(action, attr)
    if value:
        kwargs[model_name] = [(i, i) for i in action.choices]

def str_charfield_choices(kwargs, attr, default_kwargs, action, extra=None):
    model_name = default_kwargs['model_name']
    value = getattr(action, attr)
    if value:
        kwargs[model_name] = [(i, i.title()) for i in action.choices]

def required_default(kwargs, attr, default_kwargs, action, extra=None):
    # this sets blank=False if required = True
    model_name = default_kwargs['model_name']
    value = getattr(action, attr)
    kwargs[model_name] = not value

UNIVERSAL_KWARGS = {
    'required': {'model_name': 'blank', 'callback': required_default},
    'default': {'model_name': 'default'},
    'choices': {'model_name': 'choices', 'callback': charfield_choices}
    }

TYPE_FIELDS = {
    file: {'field': 'FileField',
            'getattr_kwargs': {
                'default': {'model_name': 'default', 'callback': filetype_filefield_default},
                'required': {'model_name': 'blank', 'callback': filetype_filefield_blank}
            }},
    float: {'field': 'FloatField', 'kwargs': {}},
    None: {'field': 'CharField', 'kwargs': {'max_length': 255},
              'getattr_kwargs': {
                  'default': {'model_name': 'default', 'callback': str_charfield_default},
              },
          },
    unicode: {'field': 'CharField', 'kwargs': {'max_length': 255},
              'getattr_kwargs': {
                  'default': {'model_name': 'default', 'callback': str_charfield_default},
                  'choices': {'model_name': 'choices', 'callback': str_charfield_choices}
              },
          },
    int: {'field': 'CharField', 'kwargs': {'max_length': 255}},
    str: {'field': 'CharField', 'kwargs': {'max_length': 255},
              'getattr_kwargs': {
                  'default': {'model_name': 'default', 'callback': str_charfield_default},
                  'choices': {'model_name': 'choices', 'callback': str_charfield_choices}
              },
          },
    bool: {'field': 'BooleanField',},
    argparse.FileType: {'field': 'FileField',
                        'getattr_kwargs': {
                            'default': {'model_name': 'default', 'callback': filetype_filefield_default},
                            'required': {'model_name': 'blank', 'callback': filetype_filefield_blank}
                        }}
}

ACTION_CLASS_TO_MODEL_FIELD = {
    argparse._StoreAction: dict(TYPE_FIELDS, **{

    }),
    argparse._StoreConstAction: dict(TYPE_FIELDS, **{

    }),
    argparse._StoreTrueAction: dict(TYPE_FIELDS, **{
        None: {'field': 'BooleanField', 'kwargs': {'blank': True},},
    }),
    argparse._StoreFalseAction: dict(TYPE_FIELDS, **{
        None: {'field': 'BooleanField', 'kwargs': {'blank': True},},
    })
}

def is_upload(action):
    """Checks if this should be a user upload

    :param action:
    :return: True if this is a file we intend to upload from the user
    """
    # import pdb; pdb.set_trace();
    return 'r' in action.type._mode and (action.default is None or
                                         getattr(action.default, 'name') not in (sys.stderr.name, sys.stdout.name))

class ArgParseNode(object):
    """
        This class takes an argument parser entry and assigns it to a Django field
    """
    def __init__(self, action=None, model_field=None, class_name=''):
        try:
            model_field = copy.deepcopy(model_field)
            field = model_field['field']
            self.field_module = 'models'
            if field == 'FileField':
                self.field_module = 'djangui_fields'
                if is_upload(action):
                    field = 'DjanguiUploadFileField'
                else:
                    field = 'DjanguiOutputFileField'
            #         # it's somewhere we're saving output to
            #         model_field = copy.deepcopy(TYPE_FIELDS[str])
            #         field = model_field['field']
            #         del model_field['getattr_kwargs']
            self.kwargs = model_field.get('kwargs', {})
            self.kwargs.update({'help_text': '"{0}"'.format(utils.sanitize_string(action.help))})
            getattr_kwargs = copy.deepcopy(UNIVERSAL_KWARGS)
            getattr_kwargs.update(model_field.get('getattr_kwargs', {}))
            for attr, attr_dict in getattr_kwargs.iteritems():
                cb = attr_dict.get('callback', None)
                if cb is not None:
                    cb(self.kwargs, attr, attr_dict, action, extra={'class_name': class_name})
                else:
                    model_name = attr_dict['model_name']
                    value = getattr(action, attr)
                    self.kwargs[model_name] = value
            self.name = self.get_valid_name(action.dest)
            self.kwargs.update({'verbose_name': '"{}"'.format(action.dest)})
            self.field = field
            # if self.name == 'fasta':
            #     import pdb; pdb.set_trace();
        except:
            import traceback
            print traceback.format_exc()
            import pdb; pdb.set_trace();

    def get_valid_name(self, name):
        add = 0
        new_name = name
        while new_name in FORBIDDEN:
            add+=1
            new_name = '{0}_{1}'.format(name, add)
        return new_name

    def __unicode__(self):
        return u'{0} = {3}.{1}({2})'.format(self.name, self.field,
                                               ', '.join(['{0}={1}'.format(i,v) for i,v in self.kwargs.iteritems()]),
                                               self.field_module)

    def __str__(self):
        return str(self.__unicode__())


class ArgParseNodeBuilder(object):
    def __init__(self, script, parser, script_path):
        self.nodes = []
        self.djangui_options = {}
        # places to save files to
        self.djangui_output_defaults = {}
        self.class_name = script
        self.script_path = os.path.abspath(script_path)
        self.model_description = getattr(parser, 'description', None)
        self.script_groups = []
        non_req = set([i.dest for i in parser._get_optional_actions()])
        self.optional_nodes = set([])
        self.containers = {}
        for action in parser._actions:
            # This is the help message of argparse
            if action.default == argparse.SUPPRESS:
                continue
            container = action.container.title
            container_node = self.containers.get(container, None)
            if container_node is None:
                container_node = []
                self.containers[container] = container_node
            fields = ACTION_CLASS_TO_MODEL_FIELD.get(type(action), TYPE_FIELDS)
            field_type = fields.get(action.type)
            # print action
            if field_type is None:
                field_types = [i for i in fields.keys() if i is not None and issubclass(type(action.type), i)]
                if len(field_types) > 1:
                    field_types = [i for i in fields.keys() if i is not None and isinstance(action.type, i)]
                if len(field_types) == 1:
                    field_type = fields[field_types[0]]
                else:
                    sys.stderr.write('Error creating {0}\n. {1}'.format(action, traceback.format_exc()))
                    import pdb; pdb.set_trace();
                    continue
            node = ArgParseNode(action=action, model_field=field_type, class_name=self.class_name)
            self.nodes.append(node)
            container_node.append(node.name)
            self.djangui_options[node.name] = action.option_strings[0]
            if action.dest in non_req:
                self.optional_nodes.add(node.name)
            if node.field == 'DjanguiOutputFileField':
                try:
                    self.djangui_output_defaults[node.name] = node.kwargs.pop('djangui_output_default')
                except KeyError:
                    import pdb; pdb.set_trace();
            if node.field == 'DjanguiUploadFileField':
                pass

    def getModelDict(self):
        fields = [u'djangui_script_name = models.CharField(max_length=255, default="{0}")'.format(self.script_path)]
        fields += [str(node) for node in self.nodes]
        return {'class_name': self.class_name, 'fields': fields, 'djangui_options': self.djangui_options,
                'djangui_output_defaults': self.djangui_output_defaults,
                'djangui_model_description': self.model_description, 'optional_fields': self.optional_nodes,
                'djangui_groups': dict([(i, sorted(v)) for i,v in self.containers.iteritems()])}


from django.test import TestCase

from ..backend import utils
from ..forms import DjanguiForm

from . import factories
from . import config


class FormTestCase(TestCase):

    def test_master_form(self):
        script = factories.TranslateScriptFactory()
        form = utils.get_master_form(model=script)
        assert(isinstance(form, DjanguiForm) is True)
        utils.validate_form(form=form, data=config.SCRIPT_DATA['translate'].get('data'),
                            files=config.SCRIPT_DATA['translate'].get('files'))
        assert(form.is_valid() is True)




# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('djangui', '0005_auto_20150510_2011'),
    ]

    operations = [
        migrations.AlterField(
            model_name='scriptparameter',
            name='output_path',
            field=models.FilePathField(path=b'/home/chris/Devel/newproj/newproj/user_uploads', max_length=255, allow_files=False, recursive=True, allow_folders=True),
        ),
    ]



#!/usr/bin/env python

__author__ = 'chris'
import argparse
import sys
from PIL import Image

parser = argparse.ArgumentParser(description="Crop images")
parser.add_argument('--image', help='The image to crop', type=argparse.FileType('r'), required=True)
parser.add_argument('--left', help='The number of pixels to crop from the left', type=int, default=0)
parser.add_argument('--right', help='The number of pixels to crop from the right', type=int, default=0)
parser.add_argument('--top', help='The number of pixels to crop from the top', type=int, default=0)
parser.add_argument('--bottom', help='The number of pixels to crop from the bottom', type=int, default=0)
parser.add_argument('--save', help='Where to save the new image', type=argparse.FileType('w'), required=True)

def main():
    args = parser.parse_args()
    im = Image.open(args.image)
    width, height = im.size
    right = width-args.right
    bottom = height-args.bottom
    new = im.crop((args.left, args.top, right, bottom))
    new.save(args.save)

if __name__ == "__main__":
    sys.exit(main())



from __future__ import absolute_import
__author__ = 'chris'
from django.db import models
from ..forms import fields as djangui_form_fields

class DjanguiOutputFileField(models.FileField):
    def formfield(self, **kwargs):
        # TODO: Make this from an app that is plugged in
        defaults = {'form_class': djangui_form_fields.DjanguiOutputFileField}
        defaults.update(kwargs)
        return super(DjanguiOutputFileField, self).formfield(**defaults)

class DjanguiUploadFileField(models.FileField):
    def formfield(self, **kwargs):
        # TODO: Make this from an app that is plugged in
        defaults = {'form_class': djangui_form_fields.DjanguiUploadFileField}
        defaults.update(kwargs)
        return super(DjanguiUploadFileField, self).formfield(**defaults)


#!/usr/bin/env python

__author__ = 'chris'
import argparse
import os
import sys
import pandas as pd
import seaborn as sns

parser = argparse.ArgumentParser(description="Create a heatmap from a delimited file.")
parser.add_argument('--tsv', help='The delimited file to plot.', type=argparse.FileType('r'), required=True)
parser.add_argument('--delimiter', help='The delimiter for fields. Default: tab', type=str, default='\t')
parser.add_argument('--row', help='The column containing row to create a heatmap from. Default to first row.', type=str)
parser.add_argument('--cols', help='The columns to choose values from (separate by a comma for multiple). Default: All non-rows', type=str)

def main():
    args = parser.parse_args()
    data = pd.read_table(args.tsv, index_col=args.row if args.row else 0, sep=args.delimiter)
    if args.cols:
        try:
            data = data.loc[:,args.cols.split(',')]
        except KeyError:
            data = data.iloc[:,[int(i)-1 for i in args.cols.split(',')]]
    if len(data.columns) > 50:
        raise BaseException('Too many columns')
    data = data[:100]
    seaborn_map = sns.clustermap(data)
    seaborn_map.savefig('{}_heatmap.png'.format(os.path.split(args.tsv.name)[1]))

if __name__ == "__main__":
    sys.exit(main())



'''
Created on Dec 11, 2013

@author: Chris

Collection of functions for extracting argparse related statements from the 
client code.
'''

import ast
import _ast
from itertools import *

from . import codegen


def parse_source_file(file_name):
    """
    Parses the AST of Python file for lines containing
    references to the argparse module.

    returns the collection of ast objects found.

    Example client code:

      1. parser = ArgumentParser(desc="My help Message")
      2. parser.add_argument('filename', help="Name of the file to load")
      3. parser.add_argument('-f', '--format', help='Format of output \nOptions: ['md', 'html']
      4. args = parser.parse_args()

    Variables:
      * nodes                                     Primary syntax tree object
      *    argparse_assignments       The assignment of the ArgumentParser (line 1 in example code)
      * add_arg_assignments     Calls to add_argument() (lines 2-3 in example code)
      * parser_var_name                    The instance variable of the ArgumentParser (line 1 in example code)
      * ast_source                            The curated collection of all parser related nodes in the client code
    """
    with open(file_name, 'r') as f:
        s = f.read()

    nodes = ast.parse(s)

    module_imports = get_nodes_by_instance_type(nodes, _ast.Import)
    specific_imports = get_nodes_by_instance_type(nodes, _ast.ImportFrom)

    assignment_objs = get_nodes_by_instance_type(nodes, _ast.Assign)
    call_objects = get_nodes_by_instance_type(nodes, _ast.Call)

    argparse_assignments = get_nodes_by_containing_attr(assignment_objs, 'ArgumentParser')
    group_arg_assignments = get_nodes_by_containing_attr(assignment_objs, 'add_argument_group')
    add_arg_assignments = get_nodes_by_containing_attr(call_objects, 'add_argument')
    parse_args_assignment = get_nodes_by_containing_attr(call_objects, 'parse_args')
    # there are cases where we have custom argparsers, such as subclassing ArgumentParser. The above
    # will fail on this. However, we can use the methods known to ArgumentParser to do a duck-type like
    # approach to finding what is the arg parser
    if not argparse_assignments:
        aa_references = set([i.func.value.id for i in chain(add_arg_assignments, parse_args_assignment)])
        argparse_like_objects = [getattr(i.value.func, 'id', None) for p_ref in aa_references for i in get_nodes_by_containing_attr(assignment_objs, p_ref)]
        argparse_like_objects = filter(None, argparse_like_objects)
        argparse_assignments = [get_nodes_by_containing_attr(assignment_objs, i) for i in argparse_like_objects]
        # for now, we just choose one
        try:
            argparse_assignments = argparse_assignments[0]
        except IndexError:
            pass


    # get things that are assigned inside ArgumentParser or its methods
    argparse_assigned_variables = get_node_args_and_keywords(assignment_objs, argparse_assignments, 'ArgumentParser')
    add_arg_assigned_variables = get_node_args_and_keywords(assignment_objs, add_arg_assignments, 'add_argument')
    parse_args_assigned_variables = get_node_args_and_keywords(assignment_objs, parse_args_assignment, 'parse_args')

    ast_argparse_source = chain(
        module_imports,
        specific_imports,
        argparse_assigned_variables,
        add_arg_assigned_variables,
        parse_args_assigned_variables,
        argparse_assignments,
        group_arg_assignments,
        add_arg_assignments,
    )
    return ast_argparse_source


def read_client_module(filename):
    with open(filename, 'r') as f:
        return f.readlines()


def get_node_args_and_keywords(assigned_objs, assignments, selector=None):
    referenced_nodes = set([])
    selector_line = -1
    assignment_nodes = []
    for node in assignments:
        for i in walk_tree(node):
            if i and isinstance(i, (_ast.keyword, _ast.Name)) and 'id' in i.__dict__:
                if i.id == selector:
                    selector_line = i.lineno
                elif i.lineno == selector_line:
                    referenced_nodes.add(i.id)
    for node in assigned_objs:
        for target in node.targets:
            if getattr(target, 'id', None) in referenced_nodes:
                assignment_nodes.append(node)
    return assignment_nodes


def get_nodes_by_instance_type(nodes, object_type):
    return [node for node in walk_tree(nodes) if isinstance(node, object_type)]


def get_nodes_by_containing_attr(nodes, attr):
    return [node for node in nodes if attr in walk_tree(node)]


def walk_tree(node):
    try:
        d = node.__dict__
    except AttributeError:
        d = {}
        yield node
    for key, value in d.items():
        if isinstance(value, list):
            for val in value:
                for _ in walk_tree(val):
                    yield _
        elif 'ast' in str(type(value)):
            for _ in walk_tree(value):
                yield _
        else:
            yield value


def convert_to_python(ast_source):
    """
    Converts the ast objects back into human readable Python code
    """
    return map(codegen.to_source, ast_source)


from __future__ import absolute_import
__author__ = 'chris'
from django.forms import CharField, FileField, FilePathField
from django.forms import widgets

class DjanguiOutputFileField(FileField):
    widget = widgets.TextInput

    def __init__(self, *args, **kwargs):
        kwargs['allow_empty_file'] = True
        super(DjanguiOutputFileField, self).__init__(*args, **kwargs)

# TODO: Make a complex widget of filepathfield/filefield
class DjanguiUploadFileField(FileField):
    pass
